--[[

BossReady - Shows you if your party/raid is ready for the next encounter
Author : Levva of EU 

Version 0.01
	Initial release of Addon to Curseforge SVN
	
Version 0.02
	Change to use prefixed routine names - initial window now works
	
Version 0.03
	Fixed command line parser
	
Version 0.04
	Added DebugPrint routine
	
Version 0.05
	Added bar code - heavily influenced by Zorba's Buff Bars - thanks Zorba.
	
Version 0.06
	Finally got bar adding to main frame and intially working
	
Version 0.07
	Added code for tracking extra bars
	Changed handling to trigger on specific events and to refresh the frame
	Handles "players" in Inspect.Unit.List - probably needs filtered to only in group/raid
	
Version 0.08
	Moved default position & resized frame
	Changed display to show percentages rather than actuals
	Added all the other bars and events for them
	
Version 0.09
	Fixed issues when in a party
	Regiged the way it calculates party contribution to use groupIDs
	
Version 0.10
	Added version info and tidied up debug info
	Initial Release

Version 0.11
	Add title bars
	Split bars into Group vs Personal
	
Version 0.12
	Bumped toc file to 1.5 compatility as this feature now available
	Fixed issue with offline players
	
Version 0.13
	Tweak colours
	Adjust way defaults are loaded
	
Version 0.14
	Add buttons to health and mana bars to allow expand contract
	Refactor startup code to allow button clicks to rebuild bars
	Fix debug command
	Now calculates values per role and shows on expanded bars 
	
Version 0.15
	Fix button width and text +/- when expanded/closed
	Fixed text behind bar on reloadui
	
Version 0.16
	Changed default to not expand health & mana role bars.
	Added addon load message and help
	Added reset option
	
Version 0.20
	Added ability to drag frame around
	Moved UI code to UI module
	Set bars to round percentages same as Rift bars do. Fixes issue with different percentages by 1%
	
Version 0.21
	Fix a bug where there was a nil exception when dead and running back to corpse
	
Version 0.22
	Added Ready check bar - not 100% working as no event to trigger on ready so disabling for now
	Added trigger to update on groupmember role change
	Added initial buff processing code
	
Version 0.23
	Fixed a bug with tank health percentages.
	
Version 0.24
	Fixes for new PTS code
	Removed personal title bar if patch 1.6 as planar & vitality now works for whole group
	Planar Charges bar for groups dedicated to Azunestone GM <Imperium> of Steampike EU
		whose forum posts on being ready for raiding inspired this feature.
	Reworked debugging saved variable
	
Version 0.25
	Added initial code for tooltips on various bars
	Fix for debugging text showing in error
	
Version 0.30
	Adjust for deprecation of GetFullHeight
	Added initial locale support
	Tidied up code - thanks to Lorandii for advice on localisation
	Made tooltip singular rather than 1 per bar
	Fixed non health & mana bars rounding
	Standardised function definitions to use : apart from event functions
	Added @ non-debug code for end user files
	
Version 0.40
	Added initial buff display code
	Added minimum remaining value as parameter for buffs
	Now tracks group changes using LibUnitChange
	Tracks buff changes rather than specific add/removes
	Added support for stats buffs
	Added initial work on refreshing & displaying buff bars
	Added Str, Int, Wis, Dex buff bars
	
Version 0.41
	Remove old debug code - fixes nil error
	
Version 0.42
	Fix various nil errors
	
Version 0.43
	Fix Locale issues using locale abbreviation and not locale name
	Fix Neddra's Might index not liking having ' as part of index
	
Version 0.44
	Fix base frame height
	Add a refresh option to force refresh of bars
	Revert toc to using fixed version string as project version seems bugged
	Fixed issue on first login race condition was sometimes preventing initial refresh
	Fix issue when everyone buffed still shows old toolbar status
	Set tooltip to something meaningful when everyone buffed/ready/full charges etc
	Fix issue where buff updates cast by player weren't updating bar status
	Fix tooltip text for Str, Int, Wis, Dex using endurance tooltip text (bah cut n paste)
	Fix locale spellings
	
Version 0.45
	Typo on Vitality bar tooltip
	
Version 0.50
	Add feature to disable addon in combat on by default
	User option to keep addon visible during combat
	
Version 0.51
	Added oilstones/whetstones/powerstones etc
	Refactored buff tooltip into single loop
	
--]]

--@non-debug@
local BossReady, private = ...
--@end-non-debug@

---------------------
-- Variables
---------------------

BossReady.Event = {}
BossReady.refresh = false
BossReady.refreshBuffs = false
BossReady.expandChanged = false
BossReady.inCombat = false
local L = BossReady.L

-- Set defaults for saved variables
local default = {}
default.settings = { x = 350, y = 200, hideinCombat = true,
							expandHealth = false, expandMana = false, 
							expandBasics = true, expandBuffs = false, 
							expandAugments = true, expandStats = true,
							minimumRemaining = 300, debug = false }

---------------------
-- Utilities
---------------------

function BossReady:SetDefaults()
	BossReady.saved = {}
	for k,v in pairs(default.settings) do 
		BossReady.saved[k] = default.settings[k]
	end
end

function BossReady:CommandLine(str)
	local result = {}
	local count = 0
	for token in string.gmatch(str, "[^%s]+") do
		result[count] = token
		count = count + 1
	end
	return result[0], result[1], result[2]
end

function BossReady:DebugPrint(str)
	if str and BossReady.saved.debug then
		print(str)
	end
end

function BossReady:ForceUpdate()
	BossReady.buffsLoaded = false
	BossReady.expandChanged = true
	BossReady.refresh = true
	BossReady.refreshBuffs = true
end

---------------------
-- Slash Commands
---------------------

function BossReady.Event.SlashHandler(args)
	local cmd, arg1, arg2 = BossReady:CommandLine(args)
	if cmd == L["debug"] then
		if BossReady.saved.debug == false then
			BossReady.saved.debug = true
			print(L["Debugging info will now be displayed for BossReady."])
		else
			BossReady.saved.debug = false
			print(L["Debugging info will NOT now be displayed for BossReady."])
		end
		return
	end
	if cmd == L["combat"] then
		if BossReady.saved.hideinCombat == false then
			BossReady.saved.hideinCombat = true
			print(L["BossReady frame will be hidden in combat"])
		else
			BossReady.saved.hideinCombat = false
			print(L["BossReady frame will continue to be shown in combat"])
		end
		return
	end
	if cmd == L["show"] then
		BossReady.frame:Show()
		return
	end
	if cmd == L["hide"] then
		BossReady.frame:Hide()
		return
	end
	if cmd == L["list"] then
		BossReady:ListUnits()
		return
	end
	if cmd == "version" or cmd == L["version"] then
		print(L["BossReady version"].." : "..BossReady.version)
		return
	end
	if cmd == L["refresh"] then
		BossReady:ForceUpdate()
	end
	if cmd == L["reset"] then
		BossReady:SetDefaults()
		BossReady.frame:SetPosition(default.settings.x,default.settings.y)
		BossReady:ForceUpdate()
		print(L["BossReady default values reset"])
		return
	end
	print("")
	print("BossReady : ")
	print(L["/br show | hide - shows/hides frame"])
	print(L["/br debug - turns on/off debugging info"])
	print(L["/br version - displays the addon version number"])
	print(L["/br combat - toggles whether addon is hidden in combat"])
end

---------------------
-- Main functions
---------------------

function BossReady:ListUnits()
	local units = Inspect.Unit.List()
	local count = 1
	for unitID, unitSpecifier in pairs(units) do
		print("unit "..count.." : "..unitID.." - "..unitSpecifier)
		local unitdetail = Inspect.Unit.Detail(unitID)
		if unitdetail then
			for k,v in pairs(unitdetail) do print(k,v) end
		end
		count = count + 1
	end
end

function BossReady:Refresh()
--~     if true then return end  -- turns off refresh when debugging
	local player = Inspect.Unit.Detail("player")
	-- initial loading sometimes fails on very first time you are logging in so exit refresh
	-- if no valid player object returned
	if not player then return end  
	local playerID = Inspect.Unit.Lookup("player")
	local health = { tank = 0, heal = 0, support = 0, dps = 0 }
	local healthMax = { tank = 0, heal = 0, support = 0, dps = 0 }
	local mana = { tank = 0, heal = 0, support = 0, dps = 0 }
	local manaMax = { tank = 0, heal = 0, support = 0, dps = 0 }
	health["total"] = player.health or 0
	healthMax["total"] = player.healthMax or 0
	mana["total"] = player.mana or 0
	manaMax["total"] = player.manaMax or 0
	if player.role then
		health[player.role] = player.health or 0
		healthMax[player.role] = player.healthMax or 0
		mana[player.role] = player.mana or 0
		manaMax[player.role] = player.manaMax or 0
	end
	local vitality = player.vitality or 100
	local vitalityMax = 100
	local planar = player.planar or 0
	local planarMax = player.planarMax or 0
	local offline = 0
	local afk = 0
	local ready = 0
	local pvp = 0
	local heal = 0
	local dps = 0
	local tank = 0
	local support = 0
	local afkText = ""
	local pvpText = ""
	local offlineText = ""
	local readyText = ""
	local vitalityText = ""
	local planarText = ""
	if player.afk then 
		afk = afk + 1 
		afkText = afkText..player.name..", "
	end
	if player.pvp then 
		pvpText = pvpText..player.name..", "
		pvp = pvp + 1 
	end
	if player.ready then 
		ready = ready + 1 
	else
		readyText = readyText..player.name..", "
	end
	if vitality < 100 then
		vitalityText = vitalityText..player.name.." ("..vitality.."%), "
	end
	if planar ~= planarMax then
		planarText = planarText..player.name.." ("..planar.."/"..planarMax.."), "
	end
	local players = 1
	for id = 1, 20 do
		local unitSpecifier = string.format("group%02d", id)
		local unitID = Inspect.Unit.Lookup(unitSpecifier)
		if unitID ~= playerID then
			-- since we've already added the player don't add them again.
			unitdetail = Inspect.Unit.Detail(unitSpecifier)
			if unitdetail then
				-- need to check unit to see if its in our party/raid - doesn't seem to be any reliable means of achieving that at present
				health["total"] = health["total"] + (unitdetail.health or 0)
				healthMax["total"] = healthMax["total"] + (unitdetail.healthMax or 0)
				mana["total"] = mana["total"] + (unitdetail.mana or 0)
				manaMax["total"] = manaMax["total"] + (unitdetail.manaMax or 0)
				if unitdetail.role then
					health[unitdetail.role] = (health[unitdetail.role] or 0) + (unitdetail.health or 0)
					healthMax[unitdetail.role] = (healthMax[unitdetail.role] or 0) + (unitdetail.healthMax or 0)
					mana[unitdetail.role] = (mana[unitdetail.role] or 0) + (unitdetail.mana or 0)
					manaMax[unitdetail.role] = (manaMax[unitdetail.role] or 0) + (unitdetail.manaMax or 0)
				end
				if unitdetail.vitality then
					vitality = vitality + unitdetail.vitality
					vitalityMax = vitalityMax + 100
					if unitdetail.vitality < 100 then
						vitalityText = vitalityText..unitdetail.name.." ("..vitality.."%), "
					end
				end
				if unitdetail.planar then 
					planar = planar + unitdetail.planar 
					if unitdetail.planarMax then 
						planarMax = planarMax + unitdetail.planarMax 
						if unitdetail.planar ~= unitdetail.planarMax then
							planarText = planarText..unitdetail.name.." ("..unitdetail.planar.."/"..unitdetail.planarMax.."), "
						end
					end
				end
				if unitdetail.offline then 
					offline = offline + 1 
					offlineText = offlineText..unitdetail.name..", "
				end
				if unitdetail.afk then 
					afk = afk + 1 
					afkText = afkText..unitdetail.name..", "
				end
				if unitdetail.ready then 
					ready = ready + 1 
				else
					readyText = readyText..unitdetail.name..", "
				end
				if unitdetail.pvp then 
					pvp = pvp + 1 
					pvpText = pvpText..unitdetail.name..", "
				end
				players = players + 1
			end
		end
	end
	BossReady.HealthBar:SetBarValue(health["total"], healthMax["total"], 0.5)	
	BossReady.ManaBar:SetBarValue(mana["total"], manaMax["total"], 0.5)
	BossReady.HealthBar.tank:SetBarValue(health["tank"], healthMax["tank"], 0.5)	
	BossReady.ManaBar.tank:SetBarValue(mana["tank"], manaMax["tank"], 0.5)
	BossReady.HealthBar.heal:SetBarValue(health["heal"], healthMax["heal"], 0.5)	
	BossReady.ManaBar.heal:SetBarValue(mana["heal"], manaMax["heal"], 0.5)
	BossReady.HealthBar.support:SetBarValue(health["support"], healthMax["support"], 0.5)	
	BossReady.ManaBar.support:SetBarValue(mana["support"], manaMax["support"], 0.5)
	BossReady.HealthBar.dps:SetBarValue(health["dps"], healthMax["dps"], 0.5)	
	BossReady.ManaBar.dps:SetBarValue(mana["dps"], manaMax["dps"], 0.5)
	BossReady.OfflineBar:SetBarValue(offline, players, 0)
	if offline > 0 and offlineText ~= "" then
		BossReady.OfflineBar:SetTooltipText(L["Following players are offline"]..": "..offlineText)
	else
		BossReady.OfflineBar:SetTooltipText(L["No players are offline"])
	end
	BossReady.AFKBar:SetBarValue(afk, players, 0)
	if afk > 0 and afkText ~= "" then
		BossReady.AFKBar:SetTooltipText(L["Following players are AFK"]..": "..afkText)
	else
		BossReady.AFKBar:SetTooltipText(L["No players are AFK"])
	end
	if Event.Unit.Detail.Ready then
		BossReady.ReadyBar:SetBarValue(ready, players, 0)
		if ready < players and readyText ~= "" then
			BossReady.ReadyBar:SetTooltipText(L["Following players are not ready"]..": "..readyText)
		else
			BossReady.ReadyBar:SetTooltipText(L["All players are ready"])
		end
	end
	BossReady.PvPBar:SetBarValue(pvp, players, 0)
	if pvp > 0 and pvpText ~= "" then
		BossReady.PvPBar:SetTooltipText(L["Following players are flagged PvP"]..": "..pvpText)
	else
		BossReady.PvPBar:SetTooltipText(L["No players are flagged for PvP"])
	end
	BossReady.VitalityBar:SetBarValue(vitality, vitalityMax, 0)
	if vitality < vitalityMax and vitalityText ~= "" then
		BossReady.VitalityBar:SetTooltipText(L["Following players are not fully soul healed"]..": "..vitalityText)
	else
		BossReady.VitalityBar:SetTooltipText(L["All players fully soul healed"])
	end
	BossReady.PlanarBar:SetBarValue(planar, planarMax, 0)
	if planar < planarMax and planarText ~= "" then
		BossReady.PlanarBar:SetTooltipText(L["Following players are missing planar charges"]..": "..planarText)
	else
		BossReady.PlanarBar:SetTooltipText(L["All players have full planar charges"])
	end
end

function BossReady:SetExpandingBars()
	local barcount = 1
	if BossReady.saved.expandBasics then
		BossReady.BasicsTitle.button:SetText("-")
		BossReady.HealthBar:SetVisible(true)
		BossReady.ManaBar:SetVisible(true)
		BossReady.OfflineBar:SetVisible(true)
		BossReady.AFKBar:SetVisible(true)
		BossReady.PvPBar:SetVisible(true)
		barcount = barcount + 5
		if Event.Unit.Detail.Ready then
			BossReady.ReadyBar:SetVisible(true)
			barcount = barcount + 1
		end
		if BossReady.PlayerTitle then
			BossReady.PlayerTitle:SetVisible(true)
			barcount = barcount + 1
		end
		BossReady.VitalityBar:SetVisible(true)
		BossReady.PlanarBar:SetVisible(true)
		barcount = barcount + 2
		BossReady.BuffsTitle:SetPoint("TOPCENTER", BossReady.PlanarBar, "BOTTOMCENTER")
		if BossReady.saved.expandHealth then
			BossReady.HealthBar.button:SetText("-")
			BossReady.HealthBar.tank:SetVisible(true)
			BossReady.HealthBar.heal:SetVisible(true)
			BossReady.HealthBar.support:SetVisible(true)
			BossReady.HealthBar.dps:SetVisible(true)
			barcount = barcount + 4
			BossReady.ManaBar:SetPoint("TOPCENTER", BossReady.HealthBar.dps, "BOTTOMCENTER")
		else
			BossReady.HealthBar.button:SetText("+")
			BossReady.HealthBar.tank:SetVisible(false)
			BossReady.HealthBar.heal:SetVisible(false)
			BossReady.HealthBar.support:SetVisible(false)
			BossReady.HealthBar.dps:SetVisible(false)
			BossReady.ManaBar:SetPoint("TOPCENTER", BossReady.HealthBar, "BOTTOMCENTER")
		end
		if BossReady.saved.expandMana then
			BossReady.ManaBar.button:SetText("-")
			BossReady.ManaBar.tank:SetVisible(true)
			BossReady.ManaBar.heal:SetVisible(true)
			BossReady.ManaBar.support:SetVisible(true)
			BossReady.ManaBar.dps:SetVisible(true)
			barcount = barcount + 4
			BossReady.OfflineBar:SetPoint("TOPCENTER", BossReady.ManaBar.dps, "BOTTOMCENTER")
		else
			BossReady.HealthBar.button:SetText("+")
			BossReady.ManaBar.tank:SetVisible(false)
			BossReady.ManaBar.heal:SetVisible(false)
			BossReady.ManaBar.support:SetVisible(false)
			BossReady.ManaBar.dps:SetVisible(false)
			BossReady.OfflineBar:SetPoint("TOPCENTER", BossReady.ManaBar, "BOTTOMCENTER")
		end
	else
		BossReady.BasicsTitle.button:SetText("+")
		BossReady.HealthBar:SetVisible(false)
			BossReady.HealthBar.tank:SetVisible(false)
			BossReady.HealthBar.heal:SetVisible(false)
			BossReady.HealthBar.support:SetVisible(false)
			BossReady.HealthBar.dps:SetVisible(false)
		BossReady.ManaBar:SetVisible(false)
			BossReady.ManaBar.tank:SetVisible(false)
			BossReady.ManaBar.heal:SetVisible(false)
			BossReady.ManaBar.support:SetVisible(false)
			BossReady.ManaBar.dps:SetVisible(false)
		BossReady.OfflineBar:SetVisible(false)
		BossReady.AFKBar:SetVisible(false)
		BossReady.PvPBar:SetVisible(false)
		if Event.Unit.Detail.Ready then
			BossReady.ReadyBar:SetVisible(false)
		end
		if BossReady.PlayerTitle then
			BossReady.PlayerTitle:SetVisible(false)
		end
		BossReady.VitalityBar:SetVisible(false)
		BossReady.PlanarBar:SetVisible(false)
		BossReady.BuffsTitle:SetPoint("TOPCENTER", BossReady.BasicsTitle, "BOTTOMCENTER")
	end
	if BossReady.saved.expandBuffs then
		BossReady.BuffsTitle.button:SetText("-")
		BossReady.AugmentsTitle:SetVisible(true)
		BossReady.StatsTitle:SetVisible(true)
		barcount = barcount + 3
		if BossReady.saved.expandAugments then
			BossReady.AugmentsTitle.button:SetText("-")
			BossReady.VialsBar:SetVisible(true)
			BossReady.SlackingBar:SetVisible(true)
			BossReady.ResistVialsBar:SetVisible(true)
			BossReady.StonesBar:SetVisible(true)
			BossReady.FoodBar:SetVisible(true)
			barcount = barcount + 5
			BossReady.StatsTitle:SetPoint("TOPCENTER", BossReady.FoodBar, "BOTTOMCENTER")
		else
			BossReady.AugmentsTitle.button:SetText("+")
			BossReady.VialsBar:SetVisible(false)
			BossReady.SlackingBar:SetVisible(false)
			BossReady.ResistVialsBar:SetVisible(false)
			BossReady.StonesBar:SetVisible(false)
			BossReady.FoodBar:SetVisible(false)
			BossReady.StatsTitle:SetPoint("TOPCENTER", BossReady.AugmentsTitle, "BOTTOMCENTER")
		end
		if BossReady.saved.expandStats then
			BossReady.StatsTitle.button:SetText("-")
			BossReady.EnduranceBar:SetVisible(true)
			BossReady.StrengthBar:SetVisible(true)
			BossReady.IntelligenceBar:SetVisible(true)
			BossReady.WisdomBar:SetVisible(true)
			BossReady.DexterityBar:SetVisible(true)
			barcount = barcount + 5
		else
			BossReady.StatsTitle.button:SetText("+")
			BossReady.EnduranceBar:SetVisible(false)
			BossReady.StrengthBar:SetVisible(false)
			BossReady.IntelligenceBar:SetVisible(false)
			BossReady.WisdomBar:SetVisible(false)
			BossReady.DexterityBar:SetVisible(false)
		end
	else
		BossReady.BuffsTitle.button:SetText("+")
		BossReady.AugmentsTitle:SetVisible(false)
		BossReady.VialsBar:SetVisible(false)
		BossReady.SlackingBar:SetVisible(false)
		BossReady.ResistVialsBar:SetVisible(false)
		BossReady.StonesBar:SetVisible(false)
		BossReady.FoodBar:SetVisible(false)
		BossReady.StatsTitle:SetVisible(false)
		BossReady.EnduranceBar:SetVisible(false)
		BossReady.StrengthBar:SetVisible(false)
		BossReady.IntelligenceBar:SetVisible(false)
		BossReady.WisdomBar:SetVisible(false)
		BossReady.DexterityBar:SetVisible(false)
	end
	local barheight = BossReady.BasicsTitle:GetHeight()
	local titleheight = BossReady.frame.titleframe.bar:GetHeight()
	BossReady.frame.baseframe:SetHeight(barcount * barheight + titleheight)
end

---------------------
-- Event Handlers
---------------------

function BossReady.Event.OnUpdate()
--~     if true then return end  -- turns off refresh when debugging
	if BossReady.inCombat and BossReady.saved.hideinCombat then
		BossReady.frame:Hide()
		return
	else
		BossReady.frame:Show()
	end
	if BossReady.expandChanged then
		BossReady:SetExpandingBars()
		BossReady.expandChanged = false
	end
	if BossReady.refresh then
		BossReady:Refresh()
		BossReady.refresh = false
	end
	if not BossReady.buffsLoaded then
		BossReady:LoadAllGroupBuffs()
		BossReady.buffsLoaded = true
	end
	if BossReady.refreshBuffs then
		BossReady:RefreshBuffs()
		BossReady.refreshBuffs = false
	end
end

function BossReady.Event.SaveVariables(addon)
	if addon == "BossReady" then
		-- now copy saved group to settings so that they can be preserved on logout
		BossReady_Settings = BossReady.saved
	end	
end

function BossReady.Event.LoadVariables(addon)
	if addon == "BossReady" then
		-- first load defaults to set any new ones then override with saved settings
		BossReady:SetDefaults()
		if BossReady_Settings then
			for k,v in pairs(BossReady_Settings) do 
				BossReady.saved[k] = BossReady_Settings[k]
			end
		end
		BossReady.refresh = true
	end	
end

function BossReady.Event.Refresh()
	BossReady.refresh = true
end

function BossReady.Event.UnitAvailable()
	local player = Inspect.Unit.Detail("player")
	if player and player.combat then 
		BossReady.inCombat = true
	else
		BossReady.inCombat = false
	end
	BossReady:ForceUpdate()
end

function BossReady.Event.CombatStart()
	BossReady.inCombat = true
end

function BossReady.Event.CombatEnd()
	BossReady.inCombat = false
end

---------------------
-- Register Events
---------------------

table.insert(Event.System.Update.Begin, { BossReady.Event.OnUpdate, "BossReady", "OnUpdate" })
table.insert(Command.Slash.Register("br"), { BossReady.Event.SlashHandler, "BossReady", "SlashHandler"})
table.insert(Event.Addon.SavedVariables.Save.Begin, { BossReady.Event.SaveVariables, "BossReady", "SaveVariables" })
table.insert(Event.Addon.SavedVariables.Load.End, { BossReady.Event.LoadVariables, "BossReady", "LoadVariables" })

table.insert(Event.Unit.Available, { BossReady.Event.UnitAvailable, "BossReady", "unitAvailable"} )
table.insert(Event.Unit.Detail.Health, { BossReady.Event.Refresh, "BossReady", "refresh" })
table.insert(Event.Unit.Detail.Mana, { BossReady.Event.Refresh, "BossReady", "refresh" })
table.insert(Event.Unit.Detail.Afk, { BossReady.Event.Refresh, "BossReady", "refresh" })
table.insert(Event.Unit.Detail.HealthCap, { BossReady.Event.Refresh, "BossReady", "refresh" })
table.insert(Event.Unit.Detail.HealthMax, { BossReady.Event.Refresh, "BossReady", "refresh" })
table.insert(Event.Unit.Detail.ManaMax, { BossReady.Event.Refresh, "BossReady", "refresh" })
table.insert(Event.Unit.Detail.Offline, { BossReady.Event.Refresh, "BossReady", "refresh" })
table.insert(Event.Unit.Detail.Planar, { BossReady.Event.Refresh, "BossReady", "refresh" })
table.insert(Event.Unit.Detail.PlanarMax, { BossReady.Event.Refresh, "BossReady", "refresh" })
table.insert(Event.Unit.Detail.Pvp, { BossReady.Event.Refresh, "BossReady", "refresh" })
table.insert(Event.Unit.Detail.Vitality, { BossReady.Event.Refresh, "BossReady", "refresh" })
table.insert(Event.Unit.Detail.Role, { BossReady.Event.Refresh, "BossReady", "refresh" })
if Event.Unit.Detail.Ready then
	table.insert(Event.Unit.Detail.Ready, { BossReady.Event.Refresh, "BossReady", "refresh" })
end
table.insert(Event.System.Secure.Enter, { BossReady.Event.CombatStart, "BossReady", "combatstart" })
table.insert(Event.System.Secure.Leave, { BossReady.Event.CombatEnd, "BossReady", "combatstart" })