--@non-debug@
local BossReady, private = ...
--@end-non-debug@

local L = BossReady.L
BossReady.buffsLoaded = false
BossReady.Player = {}
BossReady.ValidBuffs = { 
	["Slacking Vials"] = { name = L["Slacking Vials"], 
								buffs = { L["Mighty Brightsurge Vial"], L["Mighty Enduring Vial"], L["Mighty Fortified Vial"], L["Mighty Manasurge Vial"], L["Mighty Powersurge Vial"], },
								},
	["Vials"] = { name = L["Vials"], 
					 buffs = { L["Heroic Brightsurge Vial"], L["Heroic Enduring Vial"], L["Heroic Fortified Vial"], L["Heroic Manasurge Vial"], L["Heroic Powersurge Vial"], 
								  L["Ancient Heroic Brightsurge Vial"], L["Ancient Heroic Enduring Vial"], L["Ancient Heroic Fortified Vial"], 
								  L["Ancient Heroic Manasurge Vial"], L["Ancient Heroic Powersurge Vial"], L["Heroic Heartshatter Vial"], L["Heroic Spiritshatter Vial"], }, 
								  missing = L["Following players are missing a vial or slacking"], present = L["All players have taken a vial"],
					},
	["ResistVials"] = { name = L["Resist Vials"], 
							  buffs = 
									{ L["Vial of Death Resistance"], L["Vial of Life Resistance"], L["Vial of Air Resistance"], 
									  L["Vial of Water Resistance"], L["Vial of Earth Resistance"], L["Vial of Fire Resistance"], }, 
							missing = L["Following players are missing a resistance vial"], present = L["All players have taken a resistance vial"],
							},
	["Food"] = { name = L["Food"], 
					buffs = { L["Drake Filet"], L["Icewatch Cupcake"], L["Gravemaker Steak"], }, 
					missing = L["Following players are missing a food buff"], present = L["All players have a food buff"],
					},
	["Stones"] = { name = L["Stones"],
						buffs = { L["Exquisite Oilstone"], L["Ancient Exquisite Oilstone"], L["Exquisite Whetstone"], L["Ancient Exquisite Whetstone"], 
									L["Burning Witchstone"], L["Ancient Burning Witchstone"], L["Burning Powerstone"], L["Ancient Burning Powerstone"], 
									L["Massive Weaponstone"], L["Ancient Massive Weaponstone"], L["Burning Spellstone"], L["Ancient Burning Spellstone"], 
									L["Burning Manastone"], L["Ancient Burning Manastone"], L["Poisonous Coating"], L["Thick Armor Plating"], L["Razor-Sharp Spikes"] },
						missing = L["Following players are missing a stone weapon buff"], present = L["All players have a stone weapon buff"],
					},	
	["Endurance"] = { name = L["Endurance"],
							buffs = { L["Neddras Might"], L["Fanfare of Vigor"], L["Noble Blessing"], L["Life of the Era"], L["Aegis of Salvation"], },
							missing = L["Following players are missing a Endurance buff"], present = L["All players are buffed with Endurance"],
							},
	["Strength"] = { name = L["Strength"],
							buffs = { L["Focus of Body"], L["Fanfare of Power"], },
							missing = L["Following players are missing a Strength buff"], present = L["All players are buffed with Strength"],
							},
	["Intelligence"] = { name = L["Intelligence"],
							buffs = { L["Flaring Intellect"], L["Fanfare of Knowledge"], },
							missing = L["Following players are missing a Intelligence buff"], present = L["All players are buffed with Intelligence"],
							},
	["Wisdom"] = { name = L["Wisdom"],
							buffs = { L["Wisdom of the Era"], L["Fanfare of Knowledge"], },
							missing = L["Following players are missing a Wisdom buff"], present = L["All players are buffed with Wisdom"],
							},
	["Dexterity"] = { name = L["Dexterity"],
							buffs = { L["Combat Preparation"], L["Fanfare of Power"], },
							missing = L["Following players are missing a Dexterity buff"], present = L["All players are buffed with Dexterity"],
							},
}

--- Vials & Food Text to look for as per post by MrFunSocks
--~ Weapon buffs -

--~ Exquisite Oilstone - 2h melee weapons
--~ Exquisite Whetstone - 1h melee weapons
--~ Burning Powerstone - Casters

--~ Other item buffs -

--~ Performance Insoles - Everyone
--~ Thick Armor Plating - Tanks
--~ Razor-Sharp Spikes - Cleric & Warrior tanks 


-- Battle Rezes - Info thanks to post by Veev
--~ Cleric
--~ Absolution - 31 pts in Justicar soul - 5 minute cooldown
--~ Life's Return - 32 pts in Sentinel soul - 10 minute cooldown

--~ Mage
--~ Soul Tether - 20 pts in Chloromancer soul - 5 minute cooldown (able to spec and remove 30-60s from cooldown)
--~ Spark of Life - 44 pts in Stormcaller soul - 10 minute cooldown 

---------------------
-- Buff Functions
---------------------

function BossReady:isBuffed(unitID, buffname)
	if not BossReady.Player[unitID] or not BossReady.Player[unitID].Buffs then
		BossReady:SetupUnitDetails(unitID)
	end
	if BossReady.Player[unitID] and BossReady.Player[unitID].Buffs then
		for k, v in pairs(BossReady.Player[unitID].Buffs) do
			if v.name == buffname then
				return true
			end
		end
	end
	return false
end

function BossReady:hasBuffCategory(unitID, buffcategory)
	for _, buffname in pairs(BossReady.ValidBuffs[buffcategory].buffs) do
		if BossReady:isBuffed(unitID, buffname) then
			return true
		end
	end
	return false
end

function BossReady:AddBuffs(unitID, buffs)	
	if buffs then
		if not BossReady.Player[unitID] or not BossReady.Player[unitID].Buffs then
			BossReady:SetupUnitDetails(unitID)
		end
		for buffID in pairs(buffs) do
			local buff = Inspect.Buff.Detail(unitID, buffID)
			if buff and not buff.debuff and (buff.remaining or 0) > BossReady.saved.minimumRemaining then
				buff.id = buffID
				table.insert(BossReady.Player[unitID].Buffs, buff)
			end
		end
	end
end

function BossReady:SetupUnitDetails(unitID)
	if not BossReady.Player[unitID] then 
		BossReady.Player[unitID] = {}
	end
	local unit = Inspect.Unit.Detail(unitID)
	if unit and unit.player then
		BossReady.Player[unitID].id = unit.id
		BossReady.Player[unitID].name = unit.name
		BossReady.Player[unitID].calling = unit.calling or "unknown"
		BossReady.Player[unitID].role = unit.role or "unknown"
		BossReady.Player[unitID].level = unit.level or 0
		BossReady.Player[unitID].Buffs = {}
		BossReady:AddBuffs(unitID, Inspect.Buff.List(unitID))
	end
end

function BossReady:RefreshBuffs()
--~ 	if true then return end -- turn on command to disable refresh to allow debugging
	local player = Inspect.Unit.Detail("player")
	-- initial loading sometimes fails on very first time you are logging in so exit refresh
	-- if no valid player object returned
	if not player then return end  
	local checks = { "Vials", "ResistVials", "Stones", "Food", "Endurance", "Strength", "Intelligence", "Wisdom", "Dexterity" } 
	local check = {}
	for _, v in pairs(checks) do
		check[v] = {}
		check[v].name = v
		check[v].count = 0
		check[v].text = ""
		check[v].bar = BossReady[v.."Bar"]
	end
	local slacking = 0
	local slackingtext = ""
	local playerID = Inspect.Unit.Lookup("player")
	if BossReady:hasBuffCategory(playerID, "Slacking Vials") then
		slacking = slacking + 1
		slackingtext = player.name..", "
	end
	for k, v in pairs(checks) do
		if BossReady:hasBuffCategory(playerID, check[v].name) then
			check[v].count = check[v].count + 1
		else
			check[v].text = check[v].text..player.name..", "
		end
	end
	local players = 1
	for id = 1, 20 do
		local unitSpecifier = string.format("group%02d", id)
		local unitID = Inspect.Unit.Lookup(unitSpecifier)
		if unitID ~= playerID then
			-- since we've already added the player don't add them again.
			unitdetail = Inspect.Unit.Detail(unitSpecifier)
			if unitdetail then
				players = players + 1
				if BossReady:hasBuffCategory(unitID, "Slacking Vials") then
					slacking = slacking + 1
					slackingtext = slackingtext .. unitdetail.name..", "
				end
				for k, v in pairs(checks) do
					if BossReady:hasBuffCategory(unitID, check[v].name) then
						check[v].count = check[v].count + 1
					else
						check[v].text = check[v].text .. unitdetail.name..", "
					end
				end
			end
		end
	end
	BossReady.SlackingBar:SetBarValue(slacking, players, 0)
	if slacking > 0 and slackingText ~= "" then
		BossReady.SlackingBar:SetTooltipText(L["Following players are using old vials"]..": "..slackingText)
	else
		BossReady.SlackingBar:SetTooltipText(L["No-one is slacking and using old vials"])
	end
	for k, v in pairs(checks) do
		check[v].bar:SetBarValue(check[v].count, players, 0)
		if check[v].count < players and check[v].text ~= "" then
			check[v].bar:SetTooltipText(BossReady.ValidBuffs[v].missing..": "..check[v].text)
		else
			check[v].bar:SetTooltipText(BossReady.ValidBuffs[v].present)
		end
	end
end

function BossReady:UnitInRaid(unitID)
	local playerID = Inspect.Unit.Lookup("player")
	if unitID == playerID then 
		return true 
	end
	for id = 1, 20 do
		local raidUnit = Inspect.Unit.Lookup(string.format("group%02d", id)) or 0
		if unitID == raidUnit then
			return true
		end
	end
	return false
end

---------------------
-- Event Handlers
---------------------

function BossReady:LoadAllGroupBuffs()
	local playerID = Inspect.Unit.Lookup("player")
	BossReady:SetupUnitDetails(playerID)
	for id = 1, 20 do
		local unitID = Inspect.Unit.Lookup(string.format("group%02d", id))
		if unitID and unitID ~= playerID then
			BossReady:SetupUnitDetails(unitID)
		end
	end
	BossReady.refreshBuffs = true
end

function BossReady.Event.BuffChanged(unitID, buffs)
	if BossReady:UnitInRaid(unitID) then
		if BossReady.Player[unitID] then	
			BossReady.Player[unitID].Buffs = {}
			BossReady:AddBuffs(unitID, Inspect.Buff.List(unitID))
		else
			BossReady:SetupUnitDetails(unitID)
		end
		BossReady.refreshBuffs = true
	end
end

function BossReady.Event.UnitChanged(units)	
	BossReady.buffsLoaded = false
	BossReady.refresh = true
	BossReady.refreshBuffs = true
end

function BossReady:RegisterUnitChanged()
	for id = 1, 20 do
		table.insert(Library.LibUnitChange.Register(string.format("group%02d", id)), { BossReady.Event.UnitChanged, "BossReady", "unitchanged" })
	end
end

---------------------
-- Register Events
---------------------

table.insert(Event.Buff.Add, { BossReady.Event.BuffChanged, "BossReady", "BuffAdded" })
table.insert(Event.Buff.Remove, { BossReady.Event.BuffChanged, "BossReady", "BuffRemoved" })
table.insert(Event.Buff.Change, { BossReady.Event.BuffChanged, "BossReady", "BuffChanged" })
