--@non-debug@
local BossReady, private = ...
--@end-non-debug@
BossReady.UIcontext = UI.CreateContext("BossReady")
BossReady.UIcontext.maxx = UIParent:GetWidth()
BossReady.UIcontext.maxy = UIParent:GetHeight()
BossReady.clickOffset = {}
BossReady.mainBars = {}
BossReady.zombieBars = {}
BossReady.lastAttach = nil
BossReady.dragActive = false

---------------------
-- Locals
---------------------

local barHeight = 24
local frameWidth = 150
local frameHeight = 180
local tooltipWidth = 300
local L = BossReady.L

---------------------
-- UI Code
---------------------

function BossReady:CreateTooltip()
	local tooltip = UI.CreateFrame("Frame", "BossReadyTooltip", BossReady.UIcontext)
	tooltip.textframe = UI.CreateFrame("Text", "BossReadyTooltipText", tooltip)
	
	tooltip:SetLayer(4)
	tooltip:SetBackgroundColor(0, 0, 0, 1)
	tooltip:SetWidth(tooltipWidth)
	
	tooltip.borderTop = UI.CreateFrame("Frame", "BossReadyTooltip_btop", tooltip)
	tooltip.borderTop:SetLayer(tooltip:GetLayer() + 1)
	tooltip.borderTop:SetBackgroundColor(0.7, 0.7, 0.4, 0.9)
	tooltip.borderTop:SetPoint("TOPLEFT", tooltip, "TOPLEFT")
	tooltip.borderTop:SetPoint("BOTTOMRIGHT", tooltip, "TOPRIGHT", 0, 2)

	tooltip.borderBottom = UI.CreateFrame("Frame", "BossReadyTooltip_bbottom", tooltip)
	tooltip.borderBottom:SetLayer(tooltip:GetLayer() + 1)
	tooltip.borderBottom:SetBackgroundColor(0.7, 0.7, 0.4, 0.9)
	tooltip.borderBottom:SetPoint("TOPLEFT", tooltip, "BOTTOMLEFT", 0, -2)
	tooltip.borderBottom:SetPoint("BOTTOMRIGHT", tooltip, "BOTTOMRIGHT")

	tooltip.borderRight = UI.CreateFrame("Frame", "BossReadyTooltip_bright", tooltip)
	tooltip.borderRight:SetLayer(tooltip:GetLayer() + 1)
	tooltip.borderRight:SetBackgroundColor(0.7, 0.7, 0.4, 0.9)
	tooltip.borderRight:SetPoint("TOPLEFT", tooltip, "TOPLEFT")
	tooltip.borderRight:SetPoint("BOTTOMRIGHT", tooltip, "BOTTOMLEFT", 2, 0)

	tooltip.borderLeft = UI.CreateFrame("Frame", "BossReadyTooltip_bleft", tooltip)
	tooltip.borderLeft:SetLayer(tooltip:GetLayer() + 1)
	tooltip.borderLeft:SetBackgroundColor(0.7, 0.7, 0.4, 0.9)
	tooltip.borderLeft:SetPoint("TOPLEFT", tooltip, "TOPRIGHT", -2, 0)
	tooltip.borderLeft:SetPoint("BOTTOMRIGHT", tooltip, "BOTTOMRIGHT")

	tooltip.textframe:SetFontSize(14)
	tooltip.textframe:SetText("-")
	tooltip.textframe:SetWordwrap(true)
	tooltip.textframe:SetVisible(false)
	tooltip.textframe:SetLayer(5)
	tooltip:SetVisible(false)
	
	return tooltip
end

function BossReady:CreateBar(name, title, isTitlebar, isButton, enableTooltip)
	bar = UI.CreateFrame("Frame", name.."Bar", BossReady.frame.baseframe)
	bar.name = name
    bar.text = UI.CreateFrame("Text", name.."Text", bar)
    bar.timer = UI.CreateFrame("Text", name.."Timer", bar)
	bar.button = UI.CreateFrame("RiftButton", name.."Button", bar)

    -- Solid background - this is the actual "bar" part of it.
    bar.solid = UI.CreateFrame("Frame", "Solid", bar)
    bar.solid:SetLayer(1)  
    
    bar.text:SetText("test")
	if not BossReady.addon then -- only use if pre patch 1.6
		bar.text:SetHeight(bar.text:GetFullHeight())  -- Set an initial height. We'll be overriding this, we just want to make sure it works.
	end

    if isTitlebar then
		bar.text:SetPoint("TOPLEFT", bar, "TOPLEFT") -- The text is pinned to the top-left corner of the bar if this is a title bar
		bar.button:SetPoint("TOPRIGHT", bar, "TOPRIGHT") -- The button is pinned to the top-left corner of the bar.
	else
		bar.button:SetPoint("TOPLEFT", bar, "TOPLEFT") -- The button is pinned to the top-left corner of the bar.
		bar.text:SetPoint("TOPLEFT", bar.button, "TOPRIGHT") -- The text is pinned to the top-right corner of the icon.
	end
	bar.button:SetPoint("BOTTOM", bar, "BOTTOM") -- Vertically, it always fills the entire bar.
	bar.button:SetText("+")
	bar.button:SetWidth(20)
	bar.button:SetLayer(2)
	bar.button:SetVisible(isButton)
 	bar:SetHeight(bar.text:GetHeight())  -- The bar is set to always be as high as the text is.
    
    bar.timer:SetPoint("TOPRIGHT", bar, "TOPRIGHT") -- The timer is pinned to the top-right corner.
    
    bar.text:SetPoint("RIGHT", bar.timer, "LEFT") -- Make sure the text doesn't overrun the timer. We'll be changing the text's height based on the contents, but we'll leave the width calculated this way.
    
    bar.timer:SetLayer(3)
	bar.text:SetLayer(2) 
	-- Set the solid bar to fill the entire buff bar.
    bar.solid:SetPoint("TOPLEFT", bar, "TOPLEFT")
    bar.solid:SetPoint("BOTTOMRIGHT", bar, "BOTTOMRIGHT")
    
	bar:SetWidth(frameWidth)
	
	if enableTooltip then
		bar.tooltiptext = ""
		
		function bar:SetTooltipText(text)
			local len = string.len(text)
			if len > 2 and string.sub(text, -2) == ", " then
				self.tooltiptext = string.sub(text, 1, len - 2)
			else
				self.tooltiptext = text
			end
		end
	
		function bar.Event:MouseIn()
			if self.tooltiptext == "" then 
				BossReady.tooltip:SetVisible(false)
			else
				BossReady.tooltip.textframe:ClearAll()
				BossReady.tooltip.textframe:SetText(self.tooltiptext)
				BossReady.tooltip.textframe:SetWidth(tooltipWidth - 16)
				local tooltipHeight = BossReady.tooltip.textframe:GetHeight() + 16
				BossReady.tooltip.textframe:SetHeight(tooltipHeight)
				local startx = self:GetLeft()
				local starty = self:GetTop()
				local offsetx = math.min(BossReady.UIcontext.maxx - tooltipWidth - startx, 80)
				local offsety = math.min(BossReady.UIcontext.maxy - tooltipHeight - starty, 10)
				BossReady.tooltip:SetPoint("TOPLEFT", self, "TOPLEFT", offsetx, offsety)
				BossReady.tooltip:SetHeight(tooltipHeight)
				BossReady.tooltip:SetVisible(true)
				BossReady.tooltip.textframe:SetPoint("TOPLEFT", BossReady.tooltip, "TOPLEFT", 8 , 8)
				BossReady.tooltip.textframe:SetVisible(true)
			end
		end
		
		function bar.Event:MouseOut()
			BossReady.tooltip:SetVisible(false)
			BossReady.tooltip.textframe:SetVisible(false)
		end
	end
	
	function bar.button.Event:LeftPress()
		local parent = self:GetParent()
		if parent.name == "BasicsBar" then
			BossReady.saved.expandBasics = not BossReady.saved.expandBasics
		elseif parent.name == "BuffsBar" then
			BossReady.saved.expandBuffs = not BossReady.saved.expandBuffs
		elseif parent.name == "AugmentsBar" then
			BossReady.saved.expandAugments = not BossReady.saved.expandAugments
		elseif parent.name == "StatsBar" then
			BossReady.saved.expandStats = not BossReady.saved.expandStats
		elseif parent.name == "HealthBar" then
			BossReady.saved.expandHealth = not BossReady.saved.expandHealth
		elseif parent.name == "ManaBar" then
			BossReady.saved.expandMana = not BossReady.saved.expandMana
		end
		BossReady.expandChanged = true
		BossReady.refresh = true
	end
	
	function bar:SetBarValue(value, maxvalue, adjust)
		maxvalue = maxvalue or 0
		if value == nil or maxvalue == 0 then
			self.solid:SetPoint("RIGHT", self, 1, nil)
			self.timer:SetVisible(false)
		else
			local percent = value / maxvalue 
			self.solid:SetPoint("RIGHT", self, percent, nil)
			self.timer:SetVisible(true)
			self.timer:SetText(string.format("%d", percent * 100 + adjust, 100).."%") -- add 0.5% to display to round same as Rift bars
			if not BossReady.addon then
				self.timer:SetWidth(self.timer:GetFullWidth())
			end
		end
	end
	
	return bar
end

function BossReady:AddBar(name, title, colour, isButton, enableTooltip)
	local bar = table.remove(BossReady.zombieBars)
	if not bar then
		bar = BossReady:CreateBar(name, title, false, isButton, enableTooltip)
	end
	table.insert(BossReady.mainBars, bar)
	
	-- Show the bar and set the data.
	bar:SetVisible(true)
	bar:SetBarValue(100,100,0)
	bar.text:SetText(title)
	bar:SetBackgroundColor(colour.r, colour.g, colour.b, 0.3)
	bar.solid:SetBackgroundColor(colour.r, colour.g, colour.b)
	if not BossReady.lastAttach then
		-- This is our first bar, so we're pinning it to the top of the frame
		bar:SetPoint("TOPLEFT", BossReady.frame.baseframe, "TOPLEFT", 0, barHeight)
	else
		-- This isn't our first bar so pin it to bottom of last bar
		bar:SetPoint("TOPCENTER", BossReady.lastAttach, "BOTTOMCENTER")
	end
    BossReady.lastAttach = bar
	return bar
end

function BossReady:AddTitleBar(name, title)
	local bar = table.remove(BossReady.zombieBars)
	if not bar then
		bar = BossReady:CreateBar(name, title, true, true, false)
	end
	table.insert(BossReady.mainBars, bar)
	
	-- Show the bar and set the data.
	bar:SetVisible(true)
	bar.text:SetText(title)
	bar.text:SetFontSize(14)
	bar.text:ResizeToText()
	
	bar.timer:SetVisible(false)
	bar:SetBackgroundColor(0, 0, 0, 0)
	bar.solid:SetBackgroundColor(0, 0, 0)
	if not BossReady.lastAttach then
		-- This is our first bar, so we're pinning it to the top of the frame
		bar:SetPoint("TOPLEFT", BossReady.frame.baseframe, "TOPLEFT", 0, barHeight)
	else
		-- This isn't our first bar so pin it to bottom of last bar
		bar:SetPoint("TOPCENTER", BossReady.lastAttach, "BOTTOMCENTER")
	end
    BossReady.lastAttach = bar
	return bar
end

function BossReady:CreateMainFrame()
	local frame = {}
	frame.parent = BossReady.UIcontext
	
	frame.x = BossReady.saved.x
	frame.y = BossReady.saved.y
	frame.width = frameWidth
	frame.height = frameHeight
	frame.name = "Boss Ready"
	frame.visible = true
	frame.alpha = 0.4
	
	frame.baseframe = UI.CreateFrame("Frame", frame.name, frame.parent)
	frame.baseframe.type = "Frame"
	frame.baseframe:SetPoint("TOPLEFT", UIParent, "TOPLEFT", frame.x, frame.y)
	frame.baseframe:SetLayer(-1)
	frame.baseframe:SetHeight(frame.height)
	frame.baseframe:SetWidth(frame.width)
	frame.baseframe:SetBackgroundColor(0.1, 0.1, 0.1, frame.alpha)
	frame.baseframe:SetVisible(frame.visible)
	
	frame.titleframe = {}
	frame.titleframe.bar = UI.CreateFrame("Frame", "br_titlebar", frame.baseframe)
	frame.titleframe.bar:SetLayer(1);
	frame.titleframe.bar:SetPoint("TOPLEFT", frame.baseframe, "TOPLEFT")
	frame.titleframe.bar:SetPoint("TOPRIGHT", frame.baseframe, "TOPRIGHT");
	frame.titleframe.bar:SetHeight(barHeight)
	frame.titleframe.bar:SetBackgroundColor(0.1, 0.1, 0.1, 0.8)
	frame.titleframe.bar.type = "Frame"
	
	frame.titleframe.text = UI.CreateFrame("Text", "br_titletext", frame.baseframe) 
	frame.titleframe.text:SetText(frame.name)
	frame.titleframe.text:SetFontSize(16)
	frame.titleframe.text:ResizeToText()
	frame.titleframe.text:SetHeight(barHeight)
	if not BossReady.addon then
		frame.titleframe.text:SetWidth(frame.titleframe.text:GetFullWidth())
	end
	frame.titleframe.text:SetLayer(2)
	frame.titleframe.text:SetPoint("TOPCENTER", frame.baseframe, "TOPCENTER")
	frame.titleframe.text.type = "Text"
	
	frame.dragframe = UI.CreateFrame("Frame", "dragFrame", frame.titleframe.bar)
	frame.dragframe:SetAllPoints()
	
	function frame:SetPosition(x, y)
		self.x = x
		self.y = y
		self.baseframe:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x, y)
	end

	function frame:Show()
		self.visible = true
		self.baseframe:SetVisible(true)
	end

	function frame:Hide()
		self.visible = false
		self.baseframe:SetVisible(false)
	end
	
	function frame.dragframe.Event:LeftDown()
		BossReady.dragActive = true
		local mouseStatus = Inspect.Mouse()
		BossReady.clickOffset.x = mouseStatus.x - BossReady.saved.x
		BossReady.clickOffset.y = mouseStatus.y - BossReady.saved.y
	end
	
	function frame.dragframe.Event:LeftUp()
		BossReady.dragActive = false
	end

	function frame.dragframe.Event:LeftUpoutside()
		BossReady.dragActive = false
	end

	function frame.dragframe.Event:MouseMove(x,y)
		if BossReady.dragActive then
			frame.x = x - BossReady.clickOffset.x
			frame.y = y - BossReady.clickOffset.y
			frame.baseframe:SetPoint("TOPLEFT", UIParent, "TOPLEFT", frame.x, frame.y)
			BossReady.saved.x = frame.x
			BossReady.saved.y = frame.y
		end
	end
	
	return frame
end

function BossReady:ClearBars()  -- Hide everything we have and start adding bars one at a time.
	for _, v in pairs(BossReady.mainBars) do
		table.insert(BossReady.zombieBars, v)
		v:SetVisible(false)
		v:SetPoint("TOPCENTER", UIParent, "TOPCENTER")
	end
  
	BossReady.mainBars = {}
	BossReady.lastAttach = nil
end

function BossReady:CreateBars()
	BossReady:ClearBars()
	BossReady.frame = BossReady:CreateMainFrame()
	BossReady.tooltip = BossReady:CreateTooltip()
	BossReady.BasicsTitle = 	BossReady:AddTitleBar("BasicsBar", L["Basics"])
	BossReady.HealthBar =		BossReady:AddBar("HealthBar", L["Health"], { r = 0.4, g = 0.6, b = 0.1 }, true, false)
		BossReady.HealthBar.tank =		BossReady:AddBar("TankHBar", L["Tank"], { r = 0.4, g = 0.6, b = 0.1 }, false, false)
		BossReady.HealthBar.heal =		BossReady:AddBar("HealHBar", L["Healer"], { r = 0.4, g = 0.6, b = 0.1 }, false, false)
		BossReady.HealthBar.support =	BossReady:AddBar("SupportHBar", L["Support"], { r = 0.4, g = 0.6, b = 0.1 }, false, false)
		BossReady.HealthBar.dps =		BossReady:AddBar("DPSHBar", L["DPS"], { r = 0.4, g = 0.6, b = 0.1 }, false, false)
	BossReady.ManaBar =		BossReady:AddBar("ManaBar", L["Mana"], { r = 0.3, g = 0.3, b = 0.7 }, true, false)
		BossReady.ManaBar.tank =		BossReady:AddBar("TankMBar", L["Tank"], { r = 0.3, g = 0.3, b = 0.7 }, false, false)
		BossReady.ManaBar.heal =			BossReady:AddBar("HealMBar", L["Healer"], { r = 0.3, g = 0.3, b = 0.7 }, false, false)
		BossReady.ManaBar.support =	BossReady:AddBar("SupportMBar", L["Support"], { r = 0.3, g = 0.3, b = 0.7 }, false, false)
		BossReady.ManaBar.dps =			BossReady:AddBar("DPSMBar", L["DPS"], { r = 0.3, g = 0.3, b = 0.7 }, false, false)
	BossReady.OfflineBar =		BossReady:AddBar("OfflineBar", L["Offline"], { r = 0.7, g = 0.7, b = 0.7 }, false, true)
	BossReady.AFKBar =			BossReady:AddBar("AFKBar", L["AFK"], { r = 0.3, g = 0.3, b = 0.3 }, false, true)
	BossReady.PvPBar =			BossReady:AddBar("PvPBar", L["PvP"], { r = 0.7, g = 0.3, b = 0.3 }, false, true)
	if Event.Unit.Detail.Ready then
		BossReady.ReadyBar =	BossReady:AddBar("ReadyBar", L["Ready"], { r = 0, g = 0.7, b = 0.0 }, false, true)
	end
	if not BossReady.addon then
		-- don't show title if v1.6 as Vitality & Planar are now group wide
		BossReady.PlayerTitle = BossReady:AddTitleBar("PersonalBar", "Personal")
	end
	BossReady.VitalityBar =		BossReady:AddBar("VitalityBar", L["Vitality"], { r = 1.0, g = 0.5, b = 0.0 }, false, true)
	BossReady.PlanarBar =		BossReady:AddBar("PlanarBar", L["Planar"], { r = 0.0, g = 0.6, b = 1.0 }, false, true)
	BossReady.BuffsTitle = BossReady:AddTitleBar("BuffsBar", L["Buffs"])
	BossReady.AugmentsTitle = BossReady:AddTitleBar("AugmentsBar", "- "..L["Vials, Stones etc"])
	BossReady.VialsBar = BossReady:AddBar("VialsBar", L["Vials"], { r = 0.7, g = 0.3, b = 0.7 }, false, true)
	BossReady.SlackingBar = BossReady:AddBar("SlackingBar", L["Slacking Vials"], { r = 0.7, g = 0.3, b = 0.7 }, false, true)
	BossReady.ResistVialsBar = BossReady:AddBar("ResistVialsBar", L["Resist Vials"], { r = 0.7, g = 0.3, b = 0.7 }, false, true)
	BossReady.StonesBar = BossReady:AddBar("StonesBar", L["Stones"], { r = 0.7, g = 0.3, b = 0.7 }, false, true)
	BossReady.FoodBar = BossReady:AddBar("FoodBar", L["Food"], { r = 0.7, g = 0.3, b = 0.7 }, false, true)
	BossReady.StatsTitle = BossReady:AddTitleBar("StatsBar", "- "..L["Stats"])
	BossReady.EnduranceBar = BossReady:AddBar("EnduranceBar", L["Endurance"], { r = 0.4, g = 0.6, b = 0.1 }, false, true) 
	BossReady.StrengthBar = BossReady:AddBar("StrengthBar", L["Strength"], { r = 0.4, g = 0.6, b = 0.1 }, false, true) 
	BossReady.IntelligenceBar = BossReady:AddBar("IntelligenceBar", L["Intelligence"], { r = 0.4, g = 0.6, b = 0.1 }, false, true) 
	BossReady.WisdomBar = BossReady:AddBar("WisdomBar", L["Wisdom"], { r = 0.4, g = 0.6, b = 0.1 }, false, true) 
	BossReady.DexterityBar = BossReady:AddBar("DexterityBar", L["Dexterity"], { r = 0.4, g = 0.6, b = 0.1 }, false, true) 
	BossReady.expandChanged = true
end

---------------------
-- Event Handlers
---------------------

function BossReady.Event.InitialSetup(addon)
	if addon == "BossReady" then
		if Inspect.Addon.Detail then
			BossReady.addon = Inspect.Addon.Detail("BossReady")
			BossReady.version = BossReady.addon.toc.Version or "version not found"
		else
			BossReady.version = "v0.51"
		end
		BossReady:CreateBars()
		BossReady:RegisterUnitChanged()
		print("BossReady "..BossReady.version.." "..L["Loaded Successfully"])
		print(L["To move the window drag it about, for other options please type /br"])
	end
end

---------------------
-- Register Events
---------------------

table.insert(Event.Addon.Load.End, { BossReady.Event.InitialSetup, "BossReady", "InitialSetup" })
